import argparse


def get_fsc_script_options(arguments):
    parser = argparse.ArgumentParser("Command line arguments for the 3D FSC script")

    parser.add_argument("halfmap1", 
                        type=str,
                        help="First half map of 3D reconstruction. MRC format. Can be masked or unmasked.",
                        metavar="HALFMAP1.MRC")

    parser.add_argument("halfmap2",
                        type=str,
                        help="Second half map of 3D reconstruction. MRC format. Can be masked or unmasked.",
                        metavar="HALFMAP2.MRC")

    parser.add_argument("fullmap",
                        type=str,
                        help="Full map of 3D reconstruction. MRC format. Can be masked or unmasked, "
                             "can be sharpened or unsharpened. ",
                        metavar="FULLMAP.MRC")

    parser.add_argument('--dir',
                        dest='working_directory',
                        default='/home/sami/Data',
                        help='Path to image files')

    parser.add_argument("--apix",
                        type=float,
                        default=1.0,
                        help="Angstrom per pixel of 3D map.",
                        metavar="FLOAT")

    parser.add_argument("--mask",
                        type=str,
                        help="If given, it would be used to mask the half maps during 3DFSC generation and analysis.",
                        metavar="MASK.MRC")

    parser.add_argument("--dthetaInDegrees",
                        type=float,
                        default=20.0,
                        help="Angle of cone to be used for 3D FSC sampling in degrees. Default is 20 degrees.",
                        metavar="FLOAT")

    parser.add_argument("--histogram",
                        type=str,
                        default="histogram",
                        help="Name of output histogram graph. No file extension required - it will automatically be "
                             "given a .pdf extension. No paths please.",
                        metavar="FILENAME")

    parser.add_argument("--FSCCutoff",
                        type=float,
                        default=0.143,
                        help="FSC cutoff criterion. 0.143 is default.",
                        metavar="FLOAT")

    parser.add_argument("--ThresholdForSphericity",
                        type=float,
                        default=0.5,
                        help="Threshold value for 3DFSC volume for calculating sphericity. 0.5 is default.",
                        metavar="FLOAT")

    parser.add_argument("--HighPassFilter",
                        type=float,
                        default=200.0,
                        help="High pass filter for thresholding in Angstrom. Prevents small dips in directional "
                             "FSCs at low spatial frequency due to noise from messing up the thresholding step. "
                             "Decrease if you see a huge wedge missing from your thresholded 3DFSC volume. "
                             "200 Angstroms is default.",
                        metavar="FLOAT")

    parser.add_argument("--Skip3DFSCGeneration",
                        action="store_true",
                        help="Allows for skipping of 3DFSC generation to directly run the analysis on a previously "
                             "generated set of results.",
                        metavar="True or False")

    parser.add_argument("--numThresholdsForSphericityCalcs",
                        type=int,
                        default=0,
                        help="calculate sphericities at different threshold cutoffs to determine sphericity deviation "
                             "across spatial frequencies. This can be useful to evaluate possible effects of "
                             "overfitting or improperly assigned orientations.",
                        metavar="INT")

    return parser.parse_args(arguments)



"""
Sami Koho - 2015 - sami.koho@gmail.com

A small utility that generates Gaussian blurred images from a
series of base images. This utility was use to create an autofocus function test
dataset.

"""

import os
import sys

from scipy import ndimage, misc


def main():

    # Check input parameters
    if len(sys.argv) < 2 or not os.path.isdir(sys.argv[1]):
        print("Please specify a path to a directory that contains the pictures")
        sys.exit(1)
    path = sys.argv[1]

    # Create output directory
    output_dir = os.path.join(path, "test_image_series")
    if not os.path.exists(output_dir):
            os.makedirs(output_dir)

    # Process every image in the input path
    for image_name in os.listdir(path):
        real_path = os.path.join(path, image_name)
        if not real_path.endswith((".jpg", ".tif", ".tiff", ".png")):
            continue

        original = misc.imread(real_path, flatten=True)
        path_parts = os.path.split(real_path)
        extension = path_parts[1].split(".")[1]
        base_name = path_parts[1].split(".")[0]

        # Save original
        output_path = os.path.join(output_dir, image_name)
        misc.imsave(output_path, original)

        # Blur with Gaussian filter, Sigma=1
        file_name = base_name + "_gaussian_1." + extension
        output_path = os.path.join(output_dir, file_name)
        misc.imsave(output_path, ndimage.gaussian_filter(original, 1))

        # Blur with Gaussian filter, Sigma=2
        file_name = base_name + "_gaussian_2." + extension
        output_path = os.path.join(output_dir, file_name)
        misc.imsave(output_path, ndimage.gaussian_filter(original, 2))

if __name__ == "__main__":
    main()



from .image import Image


class ArrayDetectorData(object):
    """
    A class to handle multi-dimensional data from an array detector.
    The data consists of Images recorded with each pixel of the detector
    array. In addition, each pixel can be split by laser gates into several
    images.
    """
    def __init__(self, detectors, gates):

        self._data_container = [[None] * detectors] * gates

        self._nDetectors = detectors
        self._nGates = gates

        # Iterator helper variables
        self._iteration_axis = 'detectors'
        self.gate_idx = 0
        self.detector_idx = 0

    # region Properties

    @property
    def ndetectors(self):
        return self._nDetectors

    @property
    def ngates(self):
        return self._nGates

    @property
    def iteration_axis(self):
        return self._iteration_axis

    @iteration_axis.setter
    def iteration_axis(self, value):
        if value != 'detectors' and value != 'gates':
            raise ValueError("Not a valid iteration axis. Please choose between "
                             "detectors or gates.")
        else:
            self._iteration_axis = value
    # endregion

    def __setitem__(self, key, value):
        assert isinstance(key, tuple) and len(key) == 2
        assert isinstance(value, Image)
        gate = key[0]
        detector = key[1]
        self._data_container[gate][detector] = value

    def __getitem__(self, item):
        assert isinstance(item, tuple) and len(item) == 2
        gate = item[0]
        detector = item[1]
        assert gate < self._nGates and detector < self._nDetectors
        return self._data_container[gate][detector]

    def __iter__(self):
        return self

    def __next__(self):
        if self.gate_idx < self._nGates and self.detector_idx < self._nDetectors:
            data = self._data_container[self.gate_idx][self.detector_idx]
            if self._iteration_axis == 'detectors':
                if self.detector_idx < (self._nDetectors - 1):
                    self.detector_idx += 1
                else:
                    self.detector_idx = 0
                    self.gate_idx += 1
            else:
                if self.gate_idx < (self._nGates < 1):
                    self.gate_idx +=1
                else:
                    self.gate_idx = 0
                    self.detector_idx += 1

            return data

        else:
            self.gate_idx = 0
            self.detector_idx = 0
            raise StopIteration

    def get_photosensor(self, photosensor):
            data = ArrayDetectorData(self.ndetectors, 1)
            for i in range(self.ndetectors):
                data[0, i] = self._data_container[photosensor][i]
            return data


#!/usr/bin/env python
# -*- python -*-

"""
File:        utils_extract_channel.py
Author:      Sami Koho (sami.koho@gmail.com)

Description:
A batch processing utility to extract a single channel from RGB
images. The results are saved into Extracted sub-folder of the
original path containing the images.
"""

import os
import sys

from miplib.data.io import read, write


def main():
    path = sys.argv[1]
    assert os.path.isdir(path), path
    channel = sys.argv[2]

    # Create output directory
    output_dir = os.path.join(path, "Extracted")
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    for image_name in os.listdir(path):
        real_path = os.path.join(path, image_name)
        if not os.path.isfile(real_path):
            continue
        image = read.get_image(real_path, channel=channel)

        save_name = "channel_" + channel + "_" + image_name
        save_path = os.path.join(path, output_dir)
        save_path = os.path.join(save_path, save_name)
        write.image(image, save_path)
        print("Saved %s to %s" % (save_name, save_path))

if __name__ == "__main__":
    main()


import numpy as np

from numpy.fft import fftn, ifftn, fftshift

from miplib.data.containers.image import Image
import miplib.processing.image as imops
import miplib.processing.ndarray as arrayops

#todo: Speed up with CUDA/Multithreading. Functions are ready in the ufuncs.py

def wiener_deconvolution(image, psf, snr=30, add_pad=0, cuda=False, normalize=False):
    assert isinstance(image, Image)
    assert isinstance(psf, Image)

    image_s = Image(image.copy(), image.spacing)
    orig_shape = image.shape

    if image.ndim != psf.ndim:
        raise ValueError("Image and psf dimensions do not match")

    if psf.spacing != image.spacing:
        psf = imops.zoom_to_spacing(psf, image.spacing)

    if add_pad != 0:
        new_shape = list(i + 2*add_pad for i in image_s.shape)
        image_s = imops.zero_pad_to_shape(image_s, new_shape)

    if psf.shape != image_s.shape:
        psf = imops.zero_pad_to_shape(psf, image_s.shape)

    psf /= psf.max()

    psf_f = fftn(fftshift(psf))

    wiener = arrayops.safe_divide(np.abs(psf_f)**2/(np.abs(psf_f)**2 + snr), psf_f)

    image_s = fftn(image_s)

    image_s = Image(np.abs(ifftn(image_s * wiener).real), image.spacing)

    return imops.remove_zero_padding(image_s, orig_shape)





"""
File: miplib_entry_point_options.py

In this file the command line argument interface is configured
for the various *miplib* entry points, that can be found in the
/bin directory.
"""

import argparse

from miplib.ui.cli.deconvolution_options import get_deconvolution_options_group
from miplib.ui.cli.fusion_options import get_fusion_options_group
from miplib.ui.cli.frc_options import get_frc_options_group
from miplib.ui.cli.psf_estimation_options import get_psf_estimation_options_group
from miplib.ui.cli.registration_options import get_registration_options_group
import miplib.analysis.image_quality.filters as filters
import miplib.ui.cli.argparse_helpers as helpers

#region Fourier Ring Correlation scripts

def get_frc_script_options(arguments):
    """ Command line options for the Fourier ring correlation script
    
    Arguments:
        arguments {tuple} -- Command line parameters as a tuple of strings, 
        typically obtained as sys.argv[1:]. But one can of course just use
        string.split(" "), if using in a notebook for example.
    
    Returns:
        [Namespace object] -- Simple class used by default by parse_args() 
        to create an object holding attributes and return it.
    """
    parser = argparse.ArgumentParser(description='Fourier ring correlation analysis',
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument('directory')
    parser.add_argument('--debug',
                        action='store_true')
    parser.add_argument('--frc-mode', choices=["two-image", "one-image"], default="one-image")
    parser.add_argument('--outdir', dest='pathout',
                        help='Select output folder where to save the log file'
                             + ' and the plots')
    parse = get_common_options_group(parser)
    parser = get_frc_options_group(parser)
    return parser.parse_args(arguments)

#endregion

#region Deconvolution scripts
def get_deconvolve_script_options(arguments):
    parser = argparse.ArgumentParser(
        description="Command line arguments for the"
                    "image Deconvolution script"
    )
    parser.add_argument('image')
    parser.add_argument('psf')
    parser = get_common_options_group(parser)
    parser = get_deconvolution_options_group(parser)
    parser = get_psf_estimation_options_group(parser)
    parser = get_frc_options_group(parser)
    return parser.parse_args(arguments)

#endregion

#region Image Scanning Microscopy reconstruction scripts

def get_ism_script_options(arguments):
    """ Command line options for the ISM reconstruction script
    
    Arguments:
        arguments {tuple} -- Command line parameters as a tuple of strings, 
        typically obtained as sys.argv[1:]. But one can of course just use
        string.split(" "), if using in a notebook for example.
    
    Returns:
        [Namespace object] -- Simple class used by default by parse_args() 
        to create an object holding attributes and return it.
    """
    parser = argparse.ArgumentParser(
        description="Command line arguments for the"
                    "ISM image reconstruction script"
    )
    parser.add_argument('directory', type=helpers.parse_is_dir)
    parser.add_argument('ism_mode', 
        choices=["reassign", "wiener", "rl", "all"], 
        default="reassign",
        help="Indicate the reassignment approach"
    )
    parser = get_common_options_group(parser)
    parser = get_registration_options_group(parser)
    parser = get_deconvolution_options_group(parser)
    parser = get_psf_estimation_options_group(parser)
    parser = get_frc_options_group(parser)
    return parser.parse_args(arguments)

#endregion

#region Multi-View Reconstruction scripts

def get_import_script_options(arguments):
    """ Import script is used in *miplib* to import data to the internal
    HDF5 file structure.

    
    Arguments:
        arguments {tuple} -- Command line parameters as a tuple of strings, 
        typically obtained as sys.argv[1:]. But one can of course just use
        string.split(" "), if using in a notebook for example.
    
    Returns:
        [Namespace object] -- Simple class used by default by parse_args() 
        to create an object holding attributes and return it.
    """
    parser = argparse.ArgumentParser(
        description="Command line arguments for the"
                    "miplib data import script."
    )
    parser.add_argument('data_dir_path')
    parser.add_argument(
        '--scales',
        type=helpers.parse_int_tuple,
        action='store'
    )
    parser.add_argument(
        '--calculate-psfs',
        dest='calculate_psfs',
        action='store_true'
    )

    parser.add_argument(
        '--copy-registration-result',
        dest='copy_registration_result',
        type=helpers.parseFromToString,
        default=-1,
    )
    parser.add_argument(
        '--normalize-inputs',
        action='store_true'
    )

    return parser.parse_args(arguments)

def get_register_script_options(arguments):
    """ Command line options for the multi-view image registration script

    
    Arguments:
        arguments {tuple} -- Command line parameters as a tuple of strings, 
        typically obtained as sys.argv[1:]. But one can of course just use
        string.split(" "), if using in a notebook for example.
    
    Returns:
        [Namespace object] -- Simple class used by default by parse_args() 
        to create an object holding attributes and return it.
    """
    parser = argparse.ArgumentParser(
        description="Command line arguments for the "
                    "miplib image registration script"
    )
    parser.add_argument(
        'data_file', 
        help="Give a path to a HDF5 file that contains the images")    
    
    parser = get_common_options_group(parser)
    parser = get_registration_options_group(parser)

    return parser.parse_args(arguments)

def get_fusion_script_options(arguments):
    """ Command line options for the multi-view image fusion script

    
    Arguments:
        arguments {tuple} -- Command line parameters as a tuple of strings, 
        typically obtained as sys.argv[1:]. But one can of course just use
        string.split(" "), if using in a notebook for example.
    
    Returns:
        [Namespace object] -- Simple class used by default by parse_args() 
        to create an object holding attributes and return it.
    """
    
    parser = argparse.ArgumentParser(
        description="Command line arguments for the"
                    "miplib image fusion script"
    )
    parser.add_argument(
        'data_file', 
        help="Give a path to a HDF5 file that contains the images")
    parser = get_common_options_group(parser)
    parser = get_fusion_options_group(parser)

    return parser.parse_args(arguments)

#endregion

#region Correlative Microscopy scripts

def get_tem_correlation_options(parser):
    assert isinstance(parser, argparse.ArgumentParser)

    group = parser.add_argument_group("TEM Correlation",
                                      "Options for STED-TEM correlation")

    # Image file path prefix

    group.add_argument(
        '--emfile', '--em',
        dest='em_image_path',
        metavar='PATH',
        default=None,
        help='Specify PATH to Electro microscope Image'
    )
    # STED image path
    group.add_argument(
        '--stedfile', '--st',
        dest='sted_image_path',
        metavar='PATH',
        default=None,
        help='Specify PATH to STED Image'
    )
    group.add_argument(
        '--register',
        action='store_true'
    )
    group.add_argument(
        '--transform',
        action='store_true'
    )
    group.add_argument(
        '--transform-path', '-t',
        dest='transform_path',
        metavar='PATH',
        help='Specify PATH to transform file'
    )
    group.add_argument(
        '--tfm-type',
        dest='tfm_type',
        choices=['rigid', 'similarity'],
        default='rigid',
        help='Define the spatial transform type to be used with registration'
    )

    return parser

def get_correlate_tem_script_options(arguments):
    """ This script is used to correlate fluoresence microscope (STED) and
    TEM images

    
    Arguments:
        arguments {tuple} -- Command line parameters as a tuple of strings, 
        typically obtained as sys.argv[1:]. But one can of course just use
        string.split(" "), if using in a notebook for example.
    
    Returns:
        [Namespace object] -- Simple class used by default by parse_args() 
        to create an object holding attributes and return it.
    """
    parser = argparse.ArgumentParser(
        description="Command line arguments for the "
                    "miplib correlative STED-TEM image registration script"
    )
    parser = get_common_options_group(parser)
    parser = get_tem_correlation_options(parser)
    parser = get_registration_options_group(parser)

    return parser.parse_args(arguments)

def get_transform_script_options(arguments):
    """ A utility script that can be used to apply a saved spatial transform
    to  an image.

    
    Arguments:
        arguments {tuple} -- Command line parameters as a tuple of strings, 
        typically obtained as sys.argv[1:]. But one can of course just use
        string.split(" "), if using in a notebook for example.
    
    Returns:
        [Namespace object] -- Simple class used by default by parse_args() 
        to create an object holding attributes and return it.
    """
    parser = argparse.ArgumentParser(
        description="Command line arguments for the"
                    " miplib image transform script"
    )
    parser = get_common_options_group(parser)

    parser.add_argument('moving_image')
    parser.add_argument('fixed_image')
    parser.add_argument('transform')
    parser.add_argument('--hdf', action='store_true')

    return parser.parse_args(arguments)

#endregion

#region Image Quality Ranking

def get_quality_script_options(arguments):
    """ Command line options for the image quality ranking script

    
    Arguments:
        arguments {tuple} -- Command line parameters as a tuple of strings, 
        typically obtained as sys.argv[1:]. But one can of course just use
        string.split(" "), if using in a notebook for example.
    
    Returns:
        [Namespace object] -- Simple class used by default by parse_args() 
        to create an object holding attributes and return it.
    """
    
    parser = argparse.ArgumentParser(
        description="Command line arguments for the "
                    "image quality ranking software"
    )

    parser.add_argument(
        "--file",
        help="Defines a path to the image files",
        default=None
    )
    parser.add_argument('--debug',
                        action='store_true')
    parser.add_argument(
        "--file-filter",
        dest="file_filter",
        default=None,
        help="Define a common string in the files to be analysed"
    )
    parser.add_argument(
        "--rgb-channel",
        help="Select which channel in an RGB image is to be used for quality"
             " analysis",
        dest="rgb_channel",
        type=int,
        choices=[0, 1, 2],
        default=1
    )
    # File filtering for batch mode processing
    parser.add_argument(
        "--average-filter",
        dest="average_filter",
        type=int,
        default=0,
        help="Analyze only images with similar amount of detail, by selecting a "
             "grayscale average pixel value threshold here"
    )
    parser.add_argument(
        "--working-directory",
        dest="working_directory",
        help="Defines the location of the working directory",
        default="/home/sami/Pictures/Quality"
    )
    parser.add_argument(
        "--mode",
        choices=["file", "directory", "analyze", "plot"],
        action="append",
        help="The argument containing the functionality of the main program"
             "You can concatenate actions by defining multiple modes in a"
             "single command, e.g. --mode=directory --mode=analyze"
    )
    # Parameters for controlling the way plot functionality works.
    parser.add_argument(
        "--result",
        default="average",
        choices=["average", "fskew", "ientropy", "fentropy", "fstd",
                 "fkurtosis", "fpw", "fmean", "icv", "meanbin"],
        help="Tell how you want the results to be calculated."
    )
    parser.add_argument(
        "--npics",
        type=int,
        default=9,
        help="Define how many images are shown in the plots"
    )

    parser = filters.get_common_options(parser)
    parser = get_frc_options_group(parser)
    return parser.parse_args(arguments)


def get_power_script_options(arguments):
    """
    Command line arguments for the power.py script that is used to calculate
    1D power spectra of images within a directory.
    """
    parser = argparse.ArgumentParser(
        description="Command line options for the power.py script that can be"
                    "used to save the power spectra of images within a "
                    "directory"
    )
    parser.add_argument(
        "--working-directory",
        dest="working_directory",
        help="Defines the location of the working directory",
        default="/home/sami/Pictures/Quality"
    )
    parser.add_argument(
        "--image-size",
        dest="image_size",
        type=int,
        default=512
    )
    parser = filters.get_common_options(parser)
    return parser.parse_args(arguments)


def get_subjective_ranking_options(arguments):
    """
    Command line arguments for the subjective.py script that can be used
    to obtain subjective opinion scores for image quality.
    """
    parser = argparse.ArgumentParser(
        description="Command line arguments for the "
                    "subjective image quality ranking"
                    "script."
    )
    parser.add_argument(
        "--working-directory",
        dest="working_directory",
        help="Defines the location of the working directory",
        default="/home/sami/Pictures/Quality"
    )

    return parser.parse_args(arguments)

#endregion

#region Common options
def get_common_options_group(parser):
    """ Common options for all the above scripts
    
    Arguments:
        parser {argparse.ArgumentParser} -- An argument parser to which
        the common options group is to be added.
    
    Returns:
        [argparse.ArgumentParser] -- The parser instance augmented with 
        the new options group.
    """
    assert isinstance(parser, argparse.ArgumentParser)
    group = parser.add_argument_group("Common",
                                      "Common Options for miplib scripts")
    group.add_argument(
        '--verbose',
        action='store_true'
    )
    group.add_argument(
        '--dir',
        dest='working_directory',
        default='/home/sami/Data',
        help='Path to image files'
    )
    group.add_argument(
        '--show-plots',
        dest='show_plots',
        action='store_true',
        help='Show summary plots of registration/fusion variables'
    )
    group.add_argument(
        '--show-image',
        dest='show_image',
        action='store_true',
        help='Show a 3D image of the fusion/registration result upon '
             'completion'
    )
    group.add_argument(
        '--scale',
        type=int,
        default=100,
        help="Define the size of images to use. By default the full size "
             "originals"
             "will be used, but it is possible to select resampled images as "
             "well"
    )

    group.add_argument(
        '--channel',
        type=int,
        default=0,
        help="Select the active color channel."
    )

    group.add_argument(
        '--jupyter',
        action='store_true',
        help='A switch to enable certain functions that only work when using'
             'Jupyter notebook to run the code.'
    )
    group.add_argument(
        '--test-drive',
        dest="test_drive",
        action='store_true',
        help="Enable certain sections of code that are used for debugging or "
             "tuning parameter values with new images"
    )

    group.add_argument(
        '--evaluate',
        dest='evaluate_results',
        action='store_true',
        help='Indicate whether you want to evaluate the registration/fusion '
             'results by eye before they are saved'
             'to the data structure.'
    )

    group.add_argument(
        '--temp-dir',
        help="Specify a custom directory for Temp data. By default it will"
             "be saved into an automatically generated directory in the "
             "system's temp file directory (/temp on *nix)",
        default=None
    )

    group.add_argument(
        '--carma-gate-idx',
        type=int,
        default=0,
        help='Carma files contain several images from various detector/laser gate'
             'combinations. Some scripts only work with single images, so one can'
             'specify a certain image in the file structure with the --carma-gate-idx'
             'and --carma-det-idx keywords.'
    )

    group.add_argument(
        '--carma-det-idx',
        type=int,
        default=0,
        help='Carma files contain several images from various detector/laser gate'
             'combinations. Some scripts only work with single images, so one can'
             'specify a certain image in the file structure with the --carma-gate-idx'
             'and --carma-det-idx keywords.'
    )

    group.add_argument(
        '--plot-size',
        type=helpers.parse_float_tuple,
        default=(2.5, 2.5),
        help='Size of the generated plots (in)'
    )
    group.add_argument(
        '--save-plots',
        action='store_true',
        help='Save some extra plots that a script may generate'
    )

    return parser
#endregion




#!/usr/bin/env python

import datetime
import os
import sys

import SimpleITK as sitk

from miplib.data.io import read as ioutils
from miplib.ui import supertomo_options
from miplib.processing import itk as itkutils


def main():
    options = supertomo_options.get_transform_script_options(sys.argv[1:])
    fixed_image = None
    moving_image = None
    transform = None

    if options.hdf:
        raise NotImplementedError("Only single image files are supported "
                                  "currently")
    else:
        # CHECK FILES
        # Check that the fixed image exists
        fixed_image_path = os.path.join(
            options.working_directory,
            options.fixed_image
        )

        if not os.path.isfile(fixed_image_path):
            raise ValueError('No such file: %s' % options.fixed_image)

        # Check that the EM image exists
        moving_image_path = os.path.join(
            options.working_directory,
            options.moving_image)

        if not os.path.isfile(moving_image_path):
            raise ValueError('No such file: %s' % options.moving_image)

        transform_path = os.path.join(
            options.working_directory,
            options.transform)

        if not os.path.isfile(transform_path):
            raise ValueError('No such file: %s' % options.transform)

        # READ FILES
        fixed_image = ioutils.get_image(fixed_image_path, return_itk=True)
        moving_image = ioutils.get_image(moving_image_path, return_itk=True)
        transform = ioutils.__itk_transform(transform_path, return_itk=True)

        transformed_image = itkutils.resample_image(moving_image,
                                                    transform,
                                                    fixed_image)

        # OUTPUT
        ##########################################################################
        output_dir = datetime.datetime.now().strftime("%Y-%m-%d") + '_supertomo_out'
        output_dir = os.path.join(options.working_directory, output_dir)

        if not os.path.exists(output_dir):
            os.makedirs(output_dir)

        # Files are named according to current time (the date will be
        # in the folder name)
        date_now = datetime.datetime.now().strftime("%H-%M-%S")

        file_name = date_now + '_transformed.mha'

        rgb_image = itkutils.make_composite_rgb_image(fixed_image,
                                                               transformed_image)

        image_path = os.path.join(output_dir, file_name)
        sitk.WriteImage(rgb_image, image_path)


if __name__ == '__main__':
    main()


import numpy as np
import miplib.processing.deconvolution.wiener as wiener

from miplib.data.containers.image_data import ImageData


def wiener_fusion(data, options, gate=0, scale=100, views=None):
    assert isinstance(data, ImageData)

    if views is None:
        views = list(range(data.get_number_of_images("registered")))

    result = np.zeros(data.get_image_size(), dtype=np.float32)
    for idx in views:
        data.set_active_image(idx, gate, scale, "registered")
        image = data.get_image()
        data.set_active_image(idx, gate, scale, "psf")
        psf = data.get_image()

        result += wiener.wiener_deconvolution(image, psf,
                                              snr=options.wiener_snr, add_pad=options.block_pad)
    return result



import datetime
import os
import tempfile

import miplib.data.io.tiffile


class TempData():

    def __init__(self, directory=None):
        if directory is None:
            self.dir = tempfile.mkdtemp('-miplib.temp.data')
        else:
            date_now = datetime.datetime.now().strftime("%y_%m_%d_")
            self.dir = '{}_supertomo_temp_data'.format(date_now)
            if not os.path.exists(self.dir):
                os.mkdir(self.dir)
        self.data_file = None

    def create_data_file(self, filename, col_names, append=False):
        data_file_name = os.path.join(self.dir, filename)
        self.data_file = RowFile(data_file_name,
                                 titles=col_names,
                                 append=append)

    def write_comment(self, comment):
        self.data_file.comment(comment)

    def write_row(self, data):
        self.data_file.write(data)

    def save_image(self, data, filename):
        image_path = os.path.join(self.dir, filename)
        miplib.data.io.tiffile.imsave(image_path, data)

    def close_data_file(self):
        self.data_file.close()

    def read_data_file(self):
        self.close_data_file()
        return self.data_file.read(with_titles=True)


class RowFile:
    """
    Represents a row file.

    The RowFile class is used for creating and reading row files.

    The format of the row file is the following:
    - row file may have a header line containg the titles of columns
    - lines starting with ``#`` are ignored as comment lines
    """

    def __init__(self, filename, titles = None, append=False):
        """
        Parameters
        ----------

        filename : str
          Path to a row file
        titles : {None, list}
          A list of column headers for writing mode.
        append : bool
          When True, new data will be appended to row file.
          Otherwise, the row file will be overwritten.
        """
        self.filename = filename
        dirname = os.path.dirname(self.filename)
        if not os.path.exists(dirname) and dirname:
            os.makedirs(dirname)
        self.file = None
        self.nof_cols = 0
        self.append = append
        self.extra_titles = ()
        if titles is not None:
            self.header(*titles)

        self.data_sep = ', '

    def __del__ (self):
        if self.file is not None:
            self.file.close()

    def header(self, *titles):
        """
        Write titles of columns to file.
        """
        data = None
        extra_titles = self.extra_titles
        if self.file is None:
            if os.path.isfile(self.filename) and self.append:
                data_file = RowFile(self.filename)
                data, data_titles = data_file.read(with_titles=True)
                data_file.close()
                if data_titles!=titles:
                    self.extra_titles = extra_titles = tuple([t for t in data_titles if t not in titles])
            self.file = open(self.filename, 'w')
            self.nof_cols = len(titles + extra_titles)
            self.comment('@,@'.join(titles + extra_titles))
            self.comment('To read data from this file, use ioc.microscope.data.RowFile(%r).read().' % (self.filename))

            if data is not None:
                for i in range(len(data[data_titles[0]])):
                    data_line = []
                    for t in titles + extra_titles:
                        if t in data_titles:
                            data_line.append(data[t][i])
                        else:
                            data_line.append(0)
                    self.write(*data_line)

    def comment (self, msg):
        """
        Write a comment to file.
        """
        if self.file is not None:
            self.file.write ('#%s\n' % msg)
            self.file.flush ()

    def write(self, *data):
        """
        Write a row of data to file.
        """
        if len (data) < self.nof_cols:
            data = data + (0, ) * (self.nof_cols - len (data))
        assert len(data) == self.nof_cols
        self.file.write(', '.join(str(i).strip('[]') for i in data) + '\n')
        self.file.flush()

    def _get_titles (self, line):
        if line.startswith('"'): # csv file header
            self.data_sep = '\t'
            return tuple([t[1:-1] for t in line.strip().split('\t')])
        return tuple([t.strip() for t in line[1:].split('@,@')])

    def read(self, with_titles = False):
        """
        Read data from a row file.

        Parameters
        ----------
        with_titles : bool
          When True, return also column titles.

        Returns
        -------
        data : dict
          A mapping of column values.
        titles : tuple
          Column titles.
        """
        f = open (self.filename, 'r')
        titles = None
        d = {}
        for line in f.readlines():
            if titles is None:
                titles = self._get_titles(line)
                for t in titles:
                    d[t] = []
                continue
            if line.startswith ('#'):
                continue
            data = line.strip().split(self.data_sep)
            for i, t in enumerate (titles):
                try:
                    v = float(data[i])
                except (IndexError,ValueError):
                    v = 0.0
                d[t].append(v)
        f.close()
        if with_titles:
            return d, titles
        return d

    def close (self):
        """
        Close row file.
        """
        if self.file is not None:
            print('Closing ',self.filename)
            self.file.close ()
            self.file = None



from numba import vectorize


@vectorize(['complex64(complex64, complex64)'], target='cuda')
def cuda_complex_div(a, b):
    """
    Implements array division on GPU

    Parameters
    ----------
    :param  a  Two Numpy arrays of the same shape, dtype=numpy.complex64
    :param  b

    Returns
    -------

    a/b

    """

    return a / b


@vectorize(['complex64(complex64, complex64)'], target='parallel')
def complex_div(a, b):
    """
    Implements array division on GPU

    Parameters
    ----------
    :param  a  Two Numpy arrays of the same shape, dtype=numpy.complex64
    :param  b

    Returns
    -------

    a/b

    """

    return a / b


@vectorize(['complex64(complex64)'], target='parallel')
def complex_squared(a):
    """
    Implements array division on GPU

    Parameters
    ----------
    :param  a  Two Numpy arrays of the same shape, dtype=numpy.complex64

    Returns
    -------

    a**2

    """

    return a**2

@vectorize(['complex64(complex64)'], target='cuda')
def complex_squared_cuda(a):
    """
    Implements array division on GPU

    Parameters
    ----------
    :param  a  Two Numpy arrays of the same shape, dtype=numpy.complex64

    Returns
    -------

    a**2

    """

    return a**2


@vectorize(['complex64(complex64)'], target='parallel')
def complex_mul(a,b):
    """
    Implements array division on GPU

    Parameters
    ----------
    :param  a  Two Numpy arrays of the same shape, dtype=numpy.complex64

    Returns
    -------

    a**2

    """

    return a*b

@vectorize(['complex64(complex64)'], target='cuda')
def complex_mul_cuda(a,b):
    """
    Implements array division on GPU

    Parameters
    ----------
    :param  a  Two Numpy arrays of the same shape, dtype=numpy.complex64

    Returns
    -------

    a**2

    """

    return a*b


"""
A program to import data into our HDF5 archive format.
All the images should be contained within a single directory
that can be either provided as a command line parameter, or
alternatively the program will try to use the current working
directory as the source directory.

The files should be named according to the following pattern:

Original images:
===============================================================================

original_scale_<scale>_index_<index>_channel_<channel>_angle_<angle>.<suffix>

<scale> is the image size, as a percentage of the raw microscope image
<index> is the ordering index of the views. The regular STED image gets
        index 0, the first rotation index 1 etc.
<channel> the color channel index, should start from zero.
<angle> is the estimated rotation angle
<suffix> can be .tif/.tiff, .mhd/.mha


Registered images:
===============================================================================

registered_scale_<scale>_index_<index>_channel_<channel>_angle_<angle>.<suffix>

<scale> is the image size, as a percentage of the raw microscope image
<index> is the ordering index of the views. The regular STED image gets
        index 0, the first rotation index 1 etc.
<channel> the color channel index, should start from zero.
<angle> is the estimated rotation angle
<suffix> can be .tif/.tiff, .mhd/.mha

Transform files:
===============================================================================
transform_scale_<scale>_index_<index>_channel_<channel>_angle_<angle>.<suffix>

The <scale>, <index> and <angle> parameters correspond to the registered
image that the transform is coupled with.

PSF images:
===============================================================================

psf_scale_<scale>_index_<index>_channel_<channel>_angle_<angle>.<suffix>

<scale> is the image size, as a percentage of the raw microscope image
<index> is the ordering index of the views. The regular STED image gets
        index 0, the first rotation index 1 etc.
<channel> the color channel index, should start from zero.
<angle> is the estimated rotation angle
<suffix> can be .tif/.tiff, .mhd/.mha


"""
import os
import sys

import numpy

from miplib.data.containers import image_data
from miplib.data.definitions import *
from miplib.data.io import read
from miplib.processing import itk as itkutils
from ..ui.cli import miplib_entry_point_options 


def main():
    options = miplib_entry_point_options.get_import_script_options(sys.argv[1:])
    directory = options.data_dir_path

    # Create a new HDF5 file. If a file exists, new data will be appended.
    file_name = input("Give a name for the HDF5 file: ")
    file_name += ".hdf5"
    data_path = os.path.join(directory, file_name)
    data = image_data.ImageData(data_path)

    # Add image files that have been named according to the correct format
    for image_name in os.listdir(directory):
        full_path = os.path.join(directory, image_name)

        if full_path.endswith((".tiff", ".tif", ".mhd", ".mha")):
            images = read.get_image(full_path)
            spacing = images.spacing
        else:
            continue

        if options.normalize_inputs:
            images = (images * (255.0/images.max())).astype(numpy.uint8)

        if not all(x in image_name for x in params_c) or not any(x in image_name for x in image_types_c):
            print("Unrecognized image name %s. Skipping it." % image_name)
            continue

        image_type = image_name.split("_scale")[0]
        scale = image_name.split("scale_")[-1].split("_index")[0]
        index = image_name.split("index_")[-1].split("_channel")[0]
        channel = image_name.split("channel_")[-1].split("_angle")[0]
        angle = image_name.split("angle_")[-1].split(".")[0]

        assert all(x.isdigit() for x in (scale, index, channel, angle))
        # data, angle, spacing, index, scale, channel, chunk_size=None

        if image_type == "original":
            data.add_original_image(images, scale, index, channel, angle, spacing)
        elif image_type == "registered":
            data.add_registered_image(images, scale, index, channel, angle, spacing)
        elif image_type == "psf":
            data.add_psf(images, scale, index, channel, angle, spacing)

    # Calculate resampled images
    if options.scales is not None:
        for scale in options.scales:
            print("Creating %s percent downsampled versions of the original images" % scale)
            data.create_rescaled_images("original", scale)

    # Add transforms for registered images.
    for transform_name in os.listdir(directory):
        if not transform_name.endswith(".txt"):
            continue

        if not all(x in transform_name for x in params_c) or not "transform" in transform_name:
            print("Unrecognized transform name %s. Skipping it." % transform_name)
            continue

        scale = transform_name.split("scale_")[-1].split("_index")[0]
        index = transform_name.split("index_")[-1].split("_channel")[0]
        channel = transform_name.split("channel_")[-1].split("_angle")[0]
        angle = transform_name.split("angle_")[-1].split(".")[0]

        full_path = os.path.join(directory, transform_name)

        # First calculate registered image if not in the data structure
        if not data.check_if_exists("registered", index, channel, scale):
            print("Resampling registered image for image nr. ", index)
            data.set_active_image(0, channel, scale, "original")
            reference = data.get_itk_image()
            data.set_active_image(index, channel, scale, "original")
            moving = data.get_itk_image()

            transform = read.__itk_transform(full_path, return_itk=True)

            registered = itkutils.resample_image(moving, transform, reference=reference)
            registered = itkutils.convert_from_itk_image(registered)
            spacing = registered.spacing

            data.add_registered_image(registered, scale, index, channel, angle, spacing)

        # The add it's transform
        transform_type, params, fixed_params = read.__itk_transform(full_path)
        data.add_transform(scale, index, channel, params, fixed_params, transform_type)

    # Calculate missing PSFs
    if options.calculate_psfs:
        data.calculate_missing_psfs()

    if options.copy_registration_result != -1:
        from_scale = options.copy_registration_result[0]
        to_scale = options.copy_registration_result[1]
        data.copy_registration_result(from_scale, to_scale)

    data.close()


if __name__ == "__main__":
    main()


import argparse

def get_registration_options_group(parser):

    assert isinstance(parser, argparse.ArgumentParser)
    group = parser.add_argument_group("Registration",
                                      "Options for image registration")
    group.add_argument(
        '--initializer-off',
        dest='initializer',
        action='store_false'
    )
    group.add_argument(
        '--reg-method',
        dest='registration_method',
        choices=['mattes', 'mean-squared-difference', 'viola-wells',
                 'correlation'],
        default='correlation',
        help='Specify registration method'
    )
    group.add_argument(
        '--two-step',
        dest='two_step_registration',
        action='store_true',
        help='Select if you want to do a two phase registration, '
             'the first being with a degraded image and the second'
             'with the high-resolution original'
    )
    group.add_argument(
        '--normalize',
        action='store_true',
        help='Choose this option if you want to normalize the intensity values'
             'before registration. Some registration methods work better with'
             'normalized intensities.'
    )
    
    group.add_argument(
        '--gaussian',
        dest='gaussian_variance',
        type=float,
        default=0.0,
        help='Define variance for Gaussian blur'
    )
    group.add_argument(
        '--dilation',
        dest='dilation_size',
        type=int,
        default=0,
        help='Define size for Grayscale dilation'
    )
    group.add_argument(
        '--mean',
        dest='mean_kernel',
        type=int,
        default=0,
        help='In case you would like to use a mean filter to smoothen the '
             'images'
             'before registration, define a kernel here'
    )
    group.add_argument(
        '--median',
        dest='median_size',
        type=int,
        default=0,
        help='Enable median filtering before registering by a non-zero kernel '
             'size'
    )

    # Mattes mutual information metric specific options
    group.add_argument(
        '--mattes-histogram-bins',
        dest='mattes_histogram_bins',
        type=int,
        default=15,
        help='Specify the number of histogram bins for Mattes '
             'Mutual Information sampling'
    )
    group.add_argument(
        '--sampling-percentage',
        dest='sampling_percentage',
        type=float,
        default=0.1,
        help='Specify the number of samples to take from each '
             'histogram bin'
    )

    # Viola Wells mutual information specific parameters
    group.add_argument(
        '--vw-fixed-sd',
        dest='vw_fixed_sd',
        type=float,
        default=0.4,
        help='Specify the fixed image SD value in Viola-Wells mutual '
             'information registration'
    )
    group.add_argument(
        '--vw-moving-sd',
        dest='vw_moving_sd',
        type=float,
        default=0.4,
        help='Specify the fixed image SD value in Viola-Wells mutual '
             'information registration'
    )
    group.add_argument(
        '--vw-samples-multiplier',
        dest='vw_samples_multiplier',
        type=float,
        default=0.2,
        help='Specify the amount of spatial samples to be used in '
             'mutual information calculations. The amount is given'
             'as a proportion of the total number of pixels in the'
             'fixed image.'
    )

    # Initializer options
    group.add_argument(
        '--set-rot-axis',
        dest='rot_axis',
        type=int,
        default=0,
        help='Specify the axis for initial rotation of the '
             'moving image'
    )
    group.add_argument(
        '--set-rotation',
        dest='set_rotation',
        type=float,
        default=1.0,
        help='Specify an estimate for initial rotation angle'
    )
    group.add_argument(
        '--set-scale',
        dest='set_scale',
        type=float,
        default=1.0,
        help='Specify the initial scale for similarity transform'
    )
    # Optimizer options
    group.add_argument(
        '--set-translation-scale',
        dest='translation_scale',
        type=float,
        default=1.0,
        help='A scaling parameter to adjust optimizer behavior'
             'effect on rotation and translation. By default'
             'the translation scale is 1000 times that of rotation'
    )
    group.add_argument(
        '--set-scaling-scale',
        dest='scaling_scale',
        type=float,
        default=10.0
    )
    group.add_argument(
        '--max-step',
        dest='max_step_length',
        type=float,
        default=0.2,
        help='Specify an estimate for initial rotation angle'
    )
    group.add_argument(
        '--min-step',
        dest='min_step_length',
        type=float,
        default=0.001,
        help='Specify an estimate for initial rotation angle'
    )
    group.add_argument(
        '--x-offset',
        dest='x_offset',
        type=float,
        default=0.0
    )
    group.add_argument(
        '--y-offset',
        dest='y_offset',
        type=float,
        default=0.0
    )
    group.add_argument(
        '--z-offset',
        dest='z_offset',
        type=float,
        default=0.0
    )

    group.add_argument(
        '--reg-max-iterations',
        dest='registration_max_iterations',
        type=int,
        default=300,
        help='Specify an estimate for initial rotation angle'
    )
    group.add_argument(
        '--reg-relax-factor',
        dest='relaxation_factor',
        type=float,
        default=0.7,
        help='Defines how quickly optmizer shortens the step size'
    )
    group.add_argument(
        '--reg-print-prog',
        dest='print_registration_progress',
        action='store_true'
    )
    group.add_argument(
        '--reg-enable-observers',
        action='store_true'
    )

    group.add_argument(
        '--reg-translate-only',
        action='store_true'
    )
    group.add_argument(
        '--use-internal-type',
        dest='use_internal_type',
        action='store_true'
    )
    group.add_argument(
        '--disable-init-moments',
        dest='moments',
        action='store_false'
    )
    group.add_argument(
        '--mask-threshold',
        dest='mask_threshold',
        type=int,
        default=30,
        help='Intensity threshold for the registration spatial mask. It '
             'defaults to 30, which works with most images.'
    )
    group.add_argument(
        '--learning-rate',
        dest='learning_rate',
        type=float,
        default=.7
    )

    return parser



import hashlib
import sys
import time

import numpy

VERBOSE = False


def concatenate_to_csv(values):
    assert isinstance(values, tuple) or \
           isinstance(values, list)

    return ",".join('%.6f' % s for s in values)


def argument_string(obj):
    if isinstance(obj, str):
        return repr(obj)
    if isinstance(obj, (int, float, complex)):
        return str(obj)
    if isinstance(obj, tuple):
        if len(obj) < 2: return '(%s,)' % (', '.join(map(argument_string, obj)))
        if len(obj) < 5: return '(%s)' % (', '.join(map(argument_string, obj)))
        return '<%s-tuple>' % (len(obj))
    if isinstance(obj, list):
        if len(obj) < 5: return '[%s]' % (', '.join(map(argument_string, obj)))
        return '<%s-list>' % (len(obj))
    if isinstance(obj, numpy.ndarray):
        return '<%s %s-array>' % (obj.dtype, obj.shape)
    if obj is None:
        return str(obj)
    return '<' + str(type(obj))[8:-2] + '>'


def time_it(func):
    """Decorator: print how long calling given function took.

    Notes
    -----
    ``iocbio.utils.VERBOSE`` must be True for this decorator to be
    effective.
    """
    if not VERBOSE:
        return func

    def new_func(*args, **kws):
        t = time.time()
        r = func(*args, **kws)
        dt = time.time() - t
        print('Calling %s(%s) -> %s took %s seconds' % \
              (func.__name__, ', '.join(map(argument_string, args)), argument_string(r), dt))
        return r

    return new_func


def format_time_string(seconds):
    m, s = divmod(seconds, 60)
    h, m = divmod(m, 60)
    return "%d:%02d:%02d" % (h, m, s)


class ProgressBar:
    """Creates a text-based progress bar.

    Call the object with the ``print`` command to see the progress bar,
    which looks something like this::

        [=======>        22%                  ]

    You may specify the progress bar's width, min and max values on
    init. For example::

      bar = ProgressBar(N)
      for i in range(N):
        print bar(i)
      print bar(N)

    References
    ----------
    http://code.activestate.com/recipes/168639/

    See also
    --------
    __init__, updateComment
    """

    def __init__(self, minValue=0, maxValue=100, totalWidth=80, prefix='',
                 show_percentage=True):
        self.show_percentage = show_percentage
        self.progBar = self.progBar_last = "[]"  # This holds the progress bar string
        self.min = minValue
        self.max = maxValue
        self.span = maxValue - minValue or 1
        self.width = totalWidth
        self.amount = 0  # When amount == max, we are 100% done
        self.start_time = self.current_time = self.prev_time = time.time()
        self.starting_amount = None
        self.updateAmount(0)  # Build progress bar string
        self.prefix = prefix
        self.comment = self.comment_last = ''

    def updateComment(self, comment):
        self.comment = comment

    def updateAmount(self, newAmount=0):
        """ Update the progress bar with the new amount (with min and max
            values set at initialization; if it is over or under, it takes the
            min or max value as a default. """
        if newAmount and self.starting_amount is None:
            self.starting_amount = newAmount
            self.starting_time = time.time()
        if newAmount < self.min: newAmount = self.min
        if newAmount > self.max: newAmount = self.max
        self.prev_amount = self.amount
        self.amount = newAmount

        # Figure out the new percent done, round to an integer
        diffFromMin = float(self.amount - self.min)
        percentDone = (diffFromMin / float(self.span)) * 100.0
        percentDone = int(round(percentDone))

        # Figure out how many hash bars the percentage should be
        allFull = self.width - 2
        numHashes = (percentDone / 100.0) * allFull
        numHashes = int(round(numHashes))

        # Build a progress bar with an arrow of equal signs; special cases for
        # empty and full

        if numHashes == 0:
            self.progBar = "[>%s]" % (' ' * (allFull - 1))
        elif numHashes == allFull:
            self.progBar = "[%s]" % ('=' * allFull)
        else:
            self.progBar = "[%s>%s]" % ('=' * (numHashes - 1),
                                        ' ' * (allFull - numHashes))

        if self.show_percentage:
            # figure out where to put the percentage, roughly centered
            percentPlace = (len(self.progBar) / 2) - len(str(percentDone))
            percentString = str(percentDone) + "%"
        else:
            percentPlace = int((len(self.progBar) / 2) - len(str(percentDone)))
            percentString = '%s/%s' % (self.amount, self.span)
        # slice the percentage into the bar
        self.progBar = ''.join([self.progBar[0:percentPlace], percentString,
                                self.progBar[percentPlace + len(percentString):]])
        if self.starting_amount is not None:
            amount_diff = self.amount - self.starting_amount
            if amount_diff:
                self.prev_time = self.current_time
                self.current_time = time.time()
                elapsed = self.current_time - self.starting_time
                eta = elapsed * (self.max - self.amount) / float(amount_diff)
                self.progBar += ' ETA:' + time_to_str(eta)

    def __str__(self):
        return str(self.progBar)

    def __call__(self, value):
        """ Updates the amount, and writes to stdout. Prints a carriage return
            first, so it will overwrite the current line in stdout."""
        self.updateAmount(value)
        if self.progBar_last == self.progBar and self.comment == self.comment_last:
            return
        print('\r', end=' ')
        sys.stdout.write(self.prefix + str(self) + str(self.comment) + ' ')
        sys.stdout.flush()
        self.progBar_last = self.progBar
        self.comment_last = self.comment


class Holder:
    """ Holds pairs ``(name, value)`` as instance attributes.

    The set of Holder pairs is extendable by

    ::

      <Holder instance>.<name> = <value>

    and the values are accessible as

    ::

      value = <Holder instance>.<name>
    """

    def __init__(self, descr):
        self._descr = descr
        self._counter = 0

    def __str__(self):
        return self._descr % (self.__dict__)

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, str(self))

    def __getattr__(self, name):
        raise AttributeError('%r instance has no attribute %r' % (self, name))

    def __setattr__(self, name, obj):
        if name not in self.__dict__ and '_counter' in self.__dict__:
            self._counter += 1
        self.__dict__[name] = obj

    def iterNameValue(self):
        for k, v in self.__dict__.items():
            if k.startswith('_'):
                continue
            yield k, v

    def copy(self, **kws):
        r = self.__class__(self._descr + ' - a copy')
        for name, value in self.iterNameValue():
            setattr(r, name, value)
        for name, value in list(kws.items()):
            setattr(r, name, value)
        return r


options = Holder('Options')

alphabet = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'


def getalpha(r):
    if r >= len(alphabet):
        return '_' + nary(r - len(alphabet), len(alphabet))
    return alphabet[r]


def nary(number, base=64):
    if isinstance(number, str):
        number = eval(number)
    n = number
    s = ''
    while n:
        n1 = n // base
        r = n - n1 * base
        n = n1
        s = getalpha(r) + s
    return s


def encode(string):
    """ Return encoded string.
    """
    return nary('0x' + hashlib.md5(string).hexdigest())


def fix_exp_str(s):
    return s.replace('e+00', '').replace('e+0', 'E').replace('e+', 'E').replace('e-0', 'E-').replace('e-', 'E-')


def float_to_str(x):
    if abs(x) >= 1000: return fix_exp_str('%.1e' % x)
    if abs(x) >= 100: return '%.0f' % x
    if abs(x) >= 10: return '%.1f' % x
    if abs(x) >= 1: return '%.2f' % x
    if abs(x) >= .1: return '%.3f' % x
    if abs(x) <= 1e-6: return fix_exp_str('%.1e' % x)
    if not x: return '0'
    return fix_exp_str('%.2e' % x)


def tostr(x):
    """ Return pretty string representation of x.

    Parameters
    ----------
    x : {tuple, float, :numpy:.float32, :numpy:.float64}
    """
    if isinstance(x, tuple):
        return tuple(map(tostr, x))
    if isinstance(x, (float, numpy.float32, numpy.float64)):
        return float_to_str(x)
    return str(x)


def time_to_str(s):
    """ Return human readable time string from seconds.

    Examples
    --------
    >>> from miplib.processing.to_string import time_to_str
    >>> print time_to_str(123000000)
    3Y10M24d10h40m
    >>> print time_to_str(1230000)
    14d5h40m
    >>> print time_to_str(1230)
    20m30.0s
    >>> print time_to_str(0.123)
    123ms
    >>> print time_to_str(0.000123)
    123us
    >>> print time_to_str(0.000000123)
    123ns

    """
    seconds_in_year = 31556925.9747  # a standard SI year
    orig_s = s
    years = int(s / (seconds_in_year))
    r = []
    if years:
        r.append('%sY' % (years))
        s -= years * (seconds_in_year)
    months = int(s / (seconds_in_year / 12.0))
    if months:
        r.append('%sM' % (months))
        s -= months * (seconds_in_year / 12.0)
    days = int(s / (60 * 60 * 24))
    if days:
        r.append('%sd' % (days))
        s -= days * 60 * 60 * 24
    hours = int(s / (60 * 60))
    if hours:
        r.append('%sh' % (hours))
        s -= hours * 60 * 60
    minutes = int(s / 60)
    if minutes:
        r.append('%sm' % (minutes))
        s -= minutes * 60
    seconds = int(s)
    if seconds:
        r.append('%.1fs' % (s))
        s -= seconds
    elif not r:
        mseconds = int(s * 1000)
        if mseconds:
            r.append('%sms' % (mseconds))
            s -= mseconds / 1000
        elif not r:
            useconds = int(s * 1000000)
            if useconds:
                r.append('%sus' % (useconds))
                s -= useconds / 1000000
            elif not r:
                nseconds = int(s * 1000000000)
                if nseconds:
                    r.append('%sns' % (nseconds))
                    s -= nseconds / 1000000000
    if not r:
        return '0'
    return ''.join(r)


time2str = time_to_str


from setuptools import setup, find_packages, Extension
import numpy

setup(
    name='miplib',
    version='1.0',
    packages=find_packages(),
    install_requires=['numpy', 'scipy', 'h5py', 'SimpleITK', 'jpype1',
                      'matplotlib', 'numba', 'pyculib', 'pandas', 'pims', 'scikit-image'],
    entry_points={
        'console_scripts': [
            'miplib.import = miplib.bin.import:main',
            'miplib.correlatem = miplib.bin.correlatem:main',
            'miplib.transform = miplib.bin.transform:main',
            'miplib.fuse = miplib.bin.fuse:main',
            'miplib.register = miplib.bin.register:main',
            'miplib.deconvolve = miplib.bin.deconvolve:main',
            'miplib.resolution = miplib.bin.resolution:main',
            'miplib.ism = miplib.bin.ism:main',
            'pyimq.main = miplib.bin.pyimq:main',
            'pyimq.util.blurseq = miplib.bin.utils.create_blur_sequence:main',
            'pyimq.util.imseq = miplib.bin.utils.create_photo_test_set:main',
            'pyimq.subjective = miplib.bin.subjective:main',
            'pyimq.power = miplib.bin.power:main'
        ]
    },
    platforms=["any"],
    url='https://github.com/sakoho81/miplib',
    download_url="https://github.com/sakoho81/miplib/archive/v1.0.tar.gz",
    license='BSD',
    author='Sami Koho',
    author_email='sami.koho@gmail.com',
    description='miplib software was created for Tomographic processing '
                'of STED super-resolution microscopy images.',
    ext_modules=[
        Extension(
            'miplib.processing.ops_ext',
            ['miplib/processing/src/ops_ext.c'],
            include_dirs=[numpy.get_include()]),
        Extension(
            'miplib.data.io._tifffile',
            ['miplib/data/io/src/tifffile.c'],
            include_dirs=[numpy.get_include()]),
        Extension(
            'miplib.psf._psf',
            ['miplib/psf/src/psf.c'],
            include_dirs=[numpy.get_include()]),
    ],
    classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'License :: OSI Approved :: BSD License',  
    'Programming Language :: Python :: 3.6',
  ]
)



"""
File:        utils.py
Author:      sami.koho@gmail.com

Description:
Various sorts of small utilities for the PyImageQuality software.
Contains all kinds of code snippets that did not find a home in
the main modules.
"""

import numpy
from scipy import ndimage


def analyze_accumulation(x, fraction):
    """
    Analyze the accumulation by starting from the end of the data.
    """
    assert 0.0 < fraction <= 1.0
    final = fraction * x.sum()
    index = 1
    while x[-index:].sum() < final:
        index += 1
    return index


def calculate_entropy(data):
    """
    Calculate the Shannon entropy for data
    """
    # Calculate histogram
    histogram = ndimage.histogram(
        data,
        data.min(),
        data.max(), 50)
    # Exclude zeros
    histogram = histogram[numpy.nonzero(histogram)]
    # Normalize histogram bins to sum to one
    histogram = histogram.astype(float) / histogram.sum()
    return -numpy.sum(histogram * numpy.log2(histogram))








#!/usr/bin/env python

"""
fusion_main.py

Copyright (c) 2014 Sami Koho  All rights reserved.

This software may be modified and distributed under the terms
of the BSD license.  See the LICENSE file for details.
"""

import datetime
import os
import sys

import SimpleITK as sitk

from miplib.processing.registration import registration
from miplib.ui.cli import miplib_entry_point_options
from miplib.processing import itk as itkutils


def main():
    options = miplib_entry_point_options.get_correlate_tem_script_options(sys.argv[1:])
    
    # SETUP
    ##########################################################################

    # Check that the STED image exists
    options.sted_image_path = os.path.join(options.working_directory,
                                         options.sted_image_path)
    if not os.path.isfile(options.sted_image_path):
        print('No such file: %s' % options.sted_image_path)
        sys.exit(1)

    # Check that the EM image exists
    options.em_image_path = os.path.join(options.working_directory,
                                           options.em_image_path)
    if not os.path.isfile(options.em_image_path):
        print('No such file: %s' % options.em_image_path)
        sys.exit(1)


        
    # Load input images
    sted_image = sitk.ReadImage(options.sted_image_path)
    em_image = sitk.ReadImage(options.em_image_path)


    # PRE-PROCESSING
    ##########################################################################
    # Save originals for possible later use
    sted_original = sted_image
    em_original = em_image

    if options.dilation_size != 0:
        print('Degrading input images with Dilation filter')
        sted_image = itkutils.grayscale_dilate_filter(
            sted_image,
            options.dilation_size
        )
        em_image = itkutils.grayscale_dilate_filter(
            em_image,
            options.dilation_size
        )

    if options.gaussian_variance != 0.0:
        print('Degrading the EM image with Gaussian blur filter')

        em_image = itkutils.gaussian_blurring_filter(
            em_image,
            options.gaussian_variance
        )
    if options.mean_kernel != 0:
        sted_image = itkutils.mean_filter(
            sted_image,
            options.mean_kernel
        )
        em_image = itkutils.mean_filter(
            em_image,
            options.mean_kernel
        )
    #todo: convert the pixel type into a PixelID enum
    if options.use_internal_type:

        sted_image = itkutils.type_cast(
            sted_image,
            options.image_type
        )
        em_image = itkutils.type_cast(
            em_image,
            options.image_type
         )
    #
    # if options.threshold > 0:
    #     sted_image = itkutils.threshold_image_filter(
    #         sted_image,
    #         options.threshold
    #     )
    #
    #     em_image = itkutils.threshold_image_filter(
    #         em_image,
    #         options.threshold
    #     )

    if options.normalize:
        print('Normalizing images')

        # Normalize
        sted_image = itkutils.normalize_image_filter(sted_image)
        em_image = itkutils.normalize_image_filter(em_image)

        if options.rescale_to_full_range:
            sted_image = itkutils.rescale_intensity(sted_image)
            em_image = itkutils.rescale_intensity(em_image)


    # REGISTRATION
    ##########################################################################

    if options.tfm_type == "rigid":
        final_transform = registration.itk_registration_rigid_2d(sted_image, em_image, options)
    elif options.tfm_type == "similarity":
        final_transform = registration.itk_registration_similarity_2d(sted_image, em_image, options)
    else:
        raise ValueError(options.tfm_type)
    em_image = itkutils.resample_image(
        em_original,
        final_transform,
        reference=sted_image
    )




    # OUTPUT
    ##########################################################################

    while True:
        keep = input("Do you want to keep the results (yes/no)? ")
        if keep in ('y', 'Y', 'yes', 'YES'):
            # Files are named according to current time (the date will be
            # in the folder name)

            # Output directory name will be automatically formatted according
            # to current date and time; e.g. 2014-02-18_supertomo_output
            output_dir = datetime.datetime.now().strftime("%Y-%m-%d") + '_clem_output'
            output_dir = os.path.join(options.working_directory, output_dir)

            if not os.path.exists(output_dir):
                os.makedirs(output_dir)

            date_now = datetime.datetime.now().strftime("%H-%M-%S")
            file_name = date_now + \
                        '-clem_registration-' + \
                        options.registration_method + \
                        '.tiff'
            file_path = os.path.join(output_dir, file_name)
            tfm_name = date_now + '_transform' + '.txt'
            tfm_path = os.path.join(output_dir, tfm_name)
            sitk.WriteTransform(final_transform, tfm_path)

            rgb_image = itkutils.make_composite_rgb_image(sted_original, em_image)
            sitk.WriteImage(rgb_image, file_path)
            print("The image was saved to %s and the transform to %s in " \
                  "the output directory %s" % (file_name, tfm_name, output_dir))
            break
        elif keep in ('n', 'N', 'no', 'No'):
            print("Exiting without saving results.")
            break
        else:
            print("Unkown command. Please state yes or no")


if __name__ == "__main__":
    main()








import numpy as np


def calculate_nearest_neighbor_distances(x_coords, y_coords):
    """
    Assumes an input of two arrays with coordinates of labels (e.g. centroids of blobs) and calculates
    the distance between each label pair.

    :param x_coords: the x coordinates
    :param y_coords: the y coordinates
    :return: Sorted list of distances (mean and std)
    """

    assert isinstance(x_coords, np.ndarray)
    assert isinstance(y_coords, np.ndarray)

    length = len(x_coords)

    x_s = np.broadcast_to(x_coords, (length,) * 2)
    y_s = np.broadcast_to(y_coords, (length,) * 2)

    distances = np.sqrt((x_s - np.transpose(x_s)) ** 2 + (y_s - np.transpose(y_s)) ** 2)

    distances = np.sort(distances, 0)

    mean_distances = np.mean(distances, axis=1)
    std_distances = np.std(distances, axis=1)

    return mean_distances, std_distances


import SimpleITK as sitk
import pims
import miplib.processing.itk as itkutils
from miplib.data.containers.image import Image
from miplib.data.io import tiffile


def image(path, image):
    """
    A wrapper for the various image writing functions. The consumers
    should only call this function

    :param path:    A full path to the image.
    :param image:   An image as :type image: numpy.ndarray or sitk.image.

    :return:
    """

    assert isinstance(image, Image)

    if path.endswith(('.tiff', '.tif')):
        __tiff(path, image, image.spacing)
    else:
        __itk_image(path, image)


def __itk_image(path, image):
    """
    A writer for ITK supported image formats.

    :param path:    A full path to the image.
    :param image:   An image as :type image: numpy.ndarray.
    :param spacing: Pixel size ZXY, as a :type spacing: list.
    """
    assert isinstance(image, Image)

    image = itkutils.convert_to_itk_image(image)
    sitk.WriteImage(image, path)


def __imagej_tiff(path, image, spacing):
    """
    Write a TIFF in ImageJ mode. May improve compatibility with
    older imageJ. I would recommend using the other one instead.
    :param path:    A full path to the image.
    :param image:   An image as :type image: numpy.ndarray.
    :param spacing: Pixel size ZXY, as a :type spacing: list.
    """
    tiffile.imsave(path,
                   image,
                   imagej=True,
                   resolution=list(1.0/x for x in spacing))


def __tiff(path, image, spacing):
    """
    Write a TIFF. Will be automatically converted into BigTIFF, if the
    file is too big for regulare TIFF definition.

    :param path:    A full path to the image.
    :param image:   An image as Numpy.ndarray.
    :param spacing: Pixel size ZXY, as a list.
    """

    if image.ndim == 3:
        image_description = "images={} slices={} unit=micron spacing={}".format(image.shape[0],
                                                                                image.shape[0],
                                                                                spacing[0])
        tiffile.imsave(path,
                       image,
                       resolution=(1.0/spacing[1], 1.0/spacing[2]),
                       metadata={'description': image_description})
    else:
        tiffile.imsave(path,
                       image,
                       imagej=True,
                       resolution=(1.0 / spacing[0], 1.0 / spacing[1]))




#!/usr/bin/env python
# -*- python -*-
"""
File: power.py
Author: Sami Koho (sami.koho@gmail.com)

Description:

A utility script for extracting 1D power spectra of all images within
a defined input directory. The spectra are saved in a single csv
file, each column denoting a single image.
"""
import datetime
import os
import sys

import numpy
import pandas

from miplib.analysis.image_quality import filters
from miplib.data.io import read
from miplib.processing import image as improc
from miplib.ui.cli import miplib_entry_point_options


def main():
    options = miplib_entry_point_options.get_power_script_options(sys.argv[1:])
    path = options.working_directory

    assert os.path.isdir(path)

    # Create output directory
    output_dir = datetime.datetime.now().strftime("%Y-%m-%d")+'_PyIQ_output'
    output_dir = os.path.join(options.working_directory, output_dir)
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    # Create output file
    date_now = datetime.datetime.now().strftime("%H-%M-%S")
    file_name = date_now + '_PyIQ_power_spectra' + '.csv'
    file_path = os.path.join(output_dir, file_name)

    csv_data = pandas.DataFrame()

    # Scan through images
    for image_in in os.listdir(path):
        if not image_in.endswith((".jpg", ".tif", ".tiff", ".png")):
            continue
        path_in = os.path.join(path, image_in)

        # Get image
        image = read.get_image(path_in, channel=options.rgb_channel)
        image = improc.crop_to_rectangle(image)

        for dim in image.shape:
            if dim != options.image_size:
                image = improc.resize(image, options.image_size)
                break

        task = filters.FrequencyQuality(image, options)
        task.calculate_power_spectrum()
        task.calculate_summed_power()

        power_spectrum = task.get_power_spectrum()

        csv_data[image_in] = power_spectrum[1]

    csv_data.insert(0, "Power", numpy.linspace(0, 1, num=len(csv_data)))
    csv_data.to_csv(file_path, index=False)


if __name__ == "__main__":
    main()



import argparse

def get_frc_options_group(parser):
    assert isinstance(parser, argparse.ArgumentParser)

    group = parser.add_argument_group("Fourier ring correlation analysis", "Options for FRC analysis")

    group.add_argument('--bin-delta', dest='d_bin', type=int, default=1,
                       help='Set thickness of the ring for FRC calculation')

    group.add_argument('--square', dest='resol_square', action='store_true',
                       help='Enable analysis only in the resolution square')

    group.add_argument('--frc-curve-fit-degree',
                       dest='frc_curve_fit_degree',
                       type=int,
                       default=8)

    group.add_argument('--resolution-threshold-curve-fit-degree',
                       dest='resolution_threshold_curve_fit_degree',
                       type=int,
                       default=3)

    group.add_argument('--resolution-threshold-criterion',
                       dest='resolution_threshold_criterion',
                       choices=['one-bit', 'half-bit', 'fixed', 'three-sigma', 'snr'],
                       default='fixed')

    group.add_argument('--resolution-threshold-value',
                       type=float,
                       default=1.0/7,
                       help="The resolution threshold value to be used when fixed threshold" \
                            "is applied")

    group.add_argument('--resolution-point-sigma',
                       type=float,
                       default=0.01,
                       help="The maximum difference between the value of the FRC and threshold"
                            "curves at the intersection. ")

    group.add_argument('--resolution-snr-value',
                       type=float,
                       default=0.25,
                       help="The target SNR value for the resolution measurement.")

    group.add_argument('--angle-delta',
                       dest='d_angle',
                       type=int,
                       default=20,
                       help="The size of angle increment in directional FSC analysis."
    )

    group.add_argument('--extract-angle-delta',
                       dest='d_extract_angle',
                       type=float,
                       default=5.0,
                       help="The size of the angle when using hollow sphere iterator."
                       )

    group.add_argument('--enable-hollow-iterator',
                       dest='hollow_iterator',
                       action='store_true',
                       help="Enable hollow iterator"
                       )

    group.add_argument('--curve-fit-min',
                       dest='min_filter',
                       action='store_true',
                       help="Enable min filtering for Correlation curve fitting. This will help"
                            "with saturation artefacts, but doesn't behave nicely with very few"
                            "data points."
                       )

    group.add_argument('--disable-hamming',
                       action='store_true')

    group.add_argument('--frc-curve-fit-type',
                       choices=['smooth-spline', 'spline', 'polynomial'],
                       default='spline')

    return parser



#!/usr/bin/env python
# -*- python -*-

"""
File:        pyimq.py
Author:      Sami Koho (sami.koho@gmail.com)

Description:
This is the main program of PyImageQualityRanking software. The
purpose of the software is to sort large microscopy image datasets
according to calculated image quality parameters. Possible
applications can be: (1) finding the best images in a dataset, or
(2) finding and discarding out-of-focus images before quantitative
analysis, for example.

The behavior of the program can be controlled by a rich command
line options interface. Please run "python pyimq.py -h" for details.

The program works in four main modes, that can be controlled by
the --mode parameter:
- file:        A single file is analyzed and the results are
            printed on the terminal screen
- directory:   All the images in a directory are analyzed and
            the results are saved in a file
- analyze:     Variables are calculated from the analysis results.
- plot:        The analysis results are ordered according to a
            selected image quality variable.
License:
The PyImageQuality software is licensed under BSD open-source license.

Copyright (c) 2015, Sami Koho, Laboratory of Biophysics, University of Turku.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above
copyright notice, this list of conditions and the following
disclaimer in the documentation and/or other materials provided
with the distribution.

THIS SOFTWARE IS PROVIDED BY COPYRIGHT HOLDER AND CONTRIBUTORS ''AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDER
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

In addition to the terms of the license, we ask to acknowledge the use
of packages in scientific articles by citing the corresponding papers:

**citation here**
"""

import csv
import datetime
import os
import sys

import pandas

from miplib.analysis.image_quality import filters
from miplib.data.io import read
from miplib.ui.cli import miplib_entry_point_options
from miplib.ui.plots.image import show_pics_from_disk


def main():
    """
    The Main program of the PyImageQualityRanking software.
    """
    options = miplib_entry_point_options.get_quality_script_options(sys.argv[1:])
    path = options.working_directory
    file_path = None
    csv_data = None

    print("Mode option is %s" % options.mode)

    if "file" in options.mode:
        # In "file" mode a single file is analyzed and the various parameter
        # values are printed on screen. This functionality is provided mainly
        # for debugging purposes.
        assert options.file is not None, "You have to specify a file with a " \
                                         "--file option"
        path = os.path.join(path, options.file)
        assert os.path.isfile(path)
        image = read.get_image(path, channel=options.rgb_channel)

        print("The shape is %s" % str(image.shape))

        task = filters.LocalImageQuality(image, options)
        task.set_smoothing_kernel_size(100)
        entropy = task.calculate_image_quality()
        task2 = filters.FrequencyQuality(image, options)
        finfo = task2.analyze_power_spectrum()

        print("SPATIAL MEASURES:")
        print("The entropy value of %s is %f" % (path, entropy))
        print("ANALYSIS OF THE POWER SPECTRUM TAIL")
        print("The mean is: %e" % finfo[0])
        print("The std is: %e" % finfo[1])
        print("The entropy is %e" % finfo[2])
        print("The threshold frequency is %f Hz" % finfo[3])
        print("Power at high frequencies %e" % finfo[4])
        print("The skewness is %f" % finfo[5])
        print("The kurtosis is %f" % finfo[6])

    if "directory" in options.mode:
        # In directory mode every image in a given directory is analyzed in a
        # single run. The analysis results are saved into a csv file.

        assert os.path.isdir(path), path

        # Create output directory
        output_dir = datetime.datetime.now().strftime("%Y-%m-%d")+'_PyIQ_output'
        output_dir = os.path.join(options.working_directory, output_dir)
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)
        # Create output file
        date_now = datetime.datetime.now().strftime("%H-%M-%S")
        file_name = date_now + '_PyIQ_out' + '.csv'
        file_path = os.path.join(output_dir, file_name)
        output_file = open(file_path, 'wt')
        output_writer = csv.writer(
            output_file, quoting=csv.QUOTE_NONNUMERIC, delimiter=",")
        output_writer.writerow(
            ("Filename", "tEntropy", "tBrenner", "fMoments", "fMean", "fSTD", "fEntropy",
             "fTh", "fMaxPw", "Skew", "Kurtosis", "MeanBin"))

        for image_name in os.listdir(path):
            if options.file_filter is None or options.file_filter in image_name:
                real_path = os.path.join(path, image_name)
                # Only process images
                if not os.path.isfile(real_path) or not real_path.endswith((".jpg", ".tif", ".tiff", ".png")):
                    continue
                # ImageJ files have particular TIFF tags that can be processed correctly
                # with the options.imagej switch
                image = read.get_image(path, channel=options.rgb_channel)

                # Only grayscale images are processed. If the input is an RGB image,
                # a channel can be chosen for processing.

                # Time series sometimes contain images of very different content: the start
                # of the series may show nearly empty (black) images, whereas at the end
                # of the series the whole field-of-view may be full of cells. Ranking such
                # dataset in a single piece may be challenging. Therefore the beginning of
                # the dataset can be separated from the end, by selecting a minimum value
                # for average grayscale pixel value here.
                if options.average_filter > 0 and image.average() < options.average_filter:
                    continue

                # Run spatial domain analysis
                task = filters.LocalImageQuality(image, options)
                task.set_smoothing_kernel_size(100)
                entropy = task.calculate_image_quality()
                # Run frequency domain analysis
                task2 = filters.FrequencyQuality(image, options)
                results = task2.analyze_power_spectrum()

                task3 = filters.SpectralMoments(image, options)
                moments = task3.calculate_spectral_moments()

                task4 = filters.BrennerImageQuality(image, options)
                brenner = task4.calculate_brenner_quality()

                # Save results
                results.insert(0, moments)
                results.insert(0, brenner)
                results.insert(0, entropy)
                results.insert(0, os.path.join(path, image_name))
                output_writer.writerow(results)

                print("Done analyzing %s" % image_name)

        output_file.close()
        print("The results were saved to %s" % file_path)

    if "analyze" in options.mode:
    # In analyze mode the previously created quality ranking variables are
    # normalized to the highest value of every given variable. In addition
    # some new parameters are calculated. The results are saved into a new
    # csv file.
        if file_path is None:
            assert options.file is not None, "You have to specify a data file" \
                                             "with the --file option"
            path = os.path.join(options.working_directory, options.file)
            print(path)
            file_path = path
            assert os.path.isfile(path), "Not a valid file %s" % path
            assert path.endswith(".csv"), "Unknown suffix %s" % path.split(".")[-1]

        csv_data = pandas.read_csv(file_path)
        csv_data["cv"] = csv_data.fSTD/csv_data.fMean
        csv_data["SpatEntNorm"] = csv_data.tEntropy/csv_data.tEntropy.max()
        csv_data["SpectMean"] = csv_data.fMean/csv_data.fMean.max()
        csv_data["SpectSTDNorm"] = csv_data.fSTD/csv_data.fSTD.max()
        csv_data["InvSpectSTDNorm"] = 1- csv_data.SpectSTDNorm
        csv_data["SpectEntNorm"] = csv_data.fEntropy/csv_data.fEntropy.max()
        csv_data["SkewNorm"] = 1 - abs(csv_data.Skew)/abs(csv_data.Skew).max()
        csv_data["KurtosisNorm"] = abs(csv_data.Kurtosis)/abs(csv_data.Kurtosis).max()
        csv_data["SpectHighPowerNorm"] = csv_data.fMaxPw/csv_data.fMaxPw.max()
        csv_data["MeanBinNorm"] = csv_data.MeanBin/csv_data.MeanBin.max()
        csv_data["BrennerNorm"] = csv_data.tBrenner/csv_data.tBrenner.max()
        csv_data["SpectMomentsNorm"] = csv_data.fMoments/csv_data.fMoments.max()

        # Create output directory
        output_dir = datetime.datetime.now().strftime("%Y-%m-%d")+'_PyIQ_output'
        output_dir = os.path.join(options.working_directory, output_dir)
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)
        date_now = datetime.datetime.now().strftime("%H-%M-%S")
        file_name = date_now + '_PyIQ_analyze_out' + '.csv'
        file_path = os.path.join(output_dir, file_name)

        csv_data.to_csv(file_path)
        print("The results were saved to %s" % file_path)

    if "plot" in options.mode:
    # With the plot option the dataset is sorted according to the desired ranking variable.
    # The changes are saved to the original csv file. In addition a plot is created to show
    # a subset of highest and lowest ranked images (the amount of images to show is
    # controlled by the options.npics parameter
        if csv_data is None:
            file_path = os.path.join(options.working_directory, options.file)
            assert os.path.isfile(file_path), "Not a valid file %s" % path
            assert path.endswith(".csv"), "Unknown suffix %s" % path.split(".")[-1]
            csv_data = pandas.read_csv(file_path)
        if options.result == "average":
            csv_data["Average"] = csv_data[["InvSpectSTDNorm", "SpatEntNorm"]].mean(axis=1)
            csv_data.sort(columns="Average", ascending=False, inplace=True)
        elif options.result == "fskew":
            csv_data.sort(columns="SkewNorm", ascending=False, inplace=True)
        elif options.result == "fentropy":
            csv_data.sort(columns="SpectEntNorm", ascending=False, inplace=True)
        elif options.result == "ientropy":
            csv_data.sort(columns="SpatEntNorm", ascending=False, inplace=True)
        elif options.result == "icv":
            csv_data.sort(columns="SpatEntNorm", ascending=False, inplace=True)
        elif options.result == "fstd":
            csv_data.sort(columns="SpectSTDNorm", ascending=False, inplace=True)
        elif options.result == "fkurtosis":
            csv_data.sort(columns="KurtosisNorm", ascending=False, inplace=True)
        elif options.result == "fpw":
            csv_data.sort(columns="SpectHighPowerNorm", ascending=False, inplace=True)
        elif options.result == "fmean":
            csv_data.sort(columns="SpectHighPowerNorm", ascending=False, inplace=True)
        elif options.result == "meanbin":
            csv_data.sort(columns="MeanBinNorm", ascending=False, inplace=True)
        else:
            print("Unknown results sorting method %s" % options.result)
            sys.exit()

        best_pics = csv_data["Filename"].head(options.npics).as_matrix()
        worst_pics = csv_data["Filename"].tail(options.npics).as_matrix()
        show_pics_from_disk(best_pics, title="BEST PICS")
        show_pics_from_disk(worst_pics, title="WORST PICS")

        csv_data.to_csv(file_path, index=False)


if __name__ == "__main__":
    main()


"""
itkutils.py

Copyright (C) 2014 Sami Koho
All rights reserved.

This software may be modified and distributed under the terms
of the BSD license.  See the LICENSE file for details.

This file contains several utilities & filters for simplified
usage of ITK (www.itk.org) modules in Python. Most of the ITK classes
have been implemented in similar manner, so it should be rather
easy to include additional filters.

"""
import SimpleITK as sitk
import numpy
import scipy

from miplib.data.containers.image import Image
from miplib.processing import converters


def convert_from_itk_image(image):
    """
    A simple conversion function from ITK:Image to a Numpy array. Please notice
    that the pixel size information gets lost in the conversion. If you want
    to conserve image information, rather use ImageStack class method in
    iocbio.io.image_stack module
    """
    assert isinstance(image, sitk.Image)
    array = sitk.GetArrayFromImage(image)
    # In ITK the order of the dimensions differs from Numpy. The array conversion
    # re-orders the dimensions, but of course the same has to be done to the spacing
    # information.
    spacing = image.GetSpacing()[::-1]

    return Image(array, spacing)


def convert_to_itk_image(image):
    assert isinstance(image, Image)
    return convert_from_numpy(image, image.spacing)


def convert_from_numpy(array, spacing):
    assert isinstance(array, numpy.ndarray)
    image = sitk.GetImageFromArray(array)
    image.SetSpacing(spacing[::-1])

    return image


def make_itk_transform(type, dims, parameters, fixed_parameters):
    """
    A function that can be used to construct a ITK spatial transform from
    known transform parameters.
    :param type:                A string that exactly matches the ITK transform
    :param dims:                Number of dimensions
                                type, eg "VerorRigid3DTransform"
    :param parameters:          The transform parameters tuple
    :param fixed_parameters:    The transform fixed parameters tuple
    :return:                    Returns an initialized ITK spatial transform.
    """
    if type == "AffineTransform":
        transform = sitk.AffineTransform(dims)
    else:
        raise NotImplementedError()

    transform.SetParameters(parameters)
    transform.SetFixedParameters(fixed_parameters)

    return transform

def get_itk_transform_parameters(transform):

    tfm_type = transform.GetName()
    params = transform.GetParameters()
    fixed_params = transform.GetFixedParameters()
    
    return tfm_type, params, fixed_params


def resample_image(image, transform, reference=None, interpolation="linear"):
    """
    Resampling filter for manipulating data volumes. This function can be
    used to transform an image module or to perform up or down sampling
    for example.

    image       =   input image object itk::Image
    transform   =   desired transform itk::Transform
    image_type  =   pixel type of the image data
    reference   =   a reference image, which can be used in resizing
                    applications, when different dimensions and or
                    spacing are desired to the output image
    """
    assert isinstance(image, sitk.Image)
    if reference is None:
        reference = image

    if interpolation == "nearest":
        interpolator = sitk.sitkNearestNeighbor
    elif interpolation == "linear":
        interpolator = sitk.sitkLinear
    elif interpolation == "Bspline":
        interpolator = sitk.sitkBSpline
    else:
        raise ValueError("Unknown interpolation type.")

    resampler = sitk.ResampleImageFilter()
    resampler.SetTransform(transform)

    resampler.SetInterpolator(interpolator)
    resampler.SetOutputPixelType(reference.GetPixelID())
    resampler.SetSize(reference.GetSize())
    resampler.SetOutputOrigin(reference.GetOrigin())
    resampler.SetOutputSpacing(reference.GetSpacing())
    resampler.SetOutputDirection(reference.GetDirection())
    resampler.SetDefaultPixelValue(0)

    return resampler.Execute(image)


def rotate_image(image, angle, axis=0, interpolation="linear"):
    """
    Rotate an image around the selected axis

    :param interpolation:
    :param image: a SimpleITK image
    :param angle: rotation angle in degrees
    :param axis:  rotation axis
    :return:
    """

    assert isinstance(image, sitk.Image)

    radians = converters.degrees_to_radians(angle)

    if image.GetDimension() == 3:
        transform = sitk.Euler3DTransform()
        rotation = [0.0, 0.0, 0.0]
        rotation[axis] = radians
        transform.SetRotation(*rotation)
    elif image.GetDimension() == 2:
        transform = sitk.Euler2DTransform()
        transform.SetAngle(radians)
    else:
        raise ValueError(image)

    transform.SetCenter(calculate_center_of_image(image))

    return resample_image(image, transform, interpolation=interpolation)


def rotate_psf(psf, transform, spacing=None, return_numpy=False):
    """
    In case, only one point-spread-function (PSF) is to be used in the image
    fusion, it needs to be rotated with the transform of the moving_image.
    The transform is generated during the registration process.

    psf             = A Numpy array, containing PSF data
    transform       = itk::VersorRigid3DTransform object
    return_numpy    = it is possible to either return the result as an
                      itk:Image, or a ImageStack.

    """
    #assert isinstance(transform, sitk.VersorRigid3DTransform)

    if isinstance(psf, numpy.ndarray):
        image = convert_from_numpy(psf, spacing)
    else:
        image = psf

    assert isinstance(image, sitk.Image)

    if isinstance(transform, sitk.AffineTransform):
        #print "Hep"

        array = numpy.array(transform.GetMatrix()).reshape(3, 3)
        rotation = scipy.linalg.polar(array, "right")[0]
        matrix = tuple(rotation.ravel())
        transform.SetMatrix(matrix)
        transform.SetTranslation((0.0, 0.0, 0.0))

    else:
        # We don't want to translate, but only rotate
        parameters = transform.GetParameters()
        parameters = tuple(0.0 if i in range(3, 6) else parameters[i] for i in range(len(parameters)))
        transform.SetParameters(parameters)

    # Find  and set center of rotation This assumes that the PSF is in
    # the centre of the volume, which should be expected, as otherwise it
    # will cause translation of details in the final image.
    imdims = image.GetSize()
    imspacing = image.GetSpacing()

    center = list(map(
        lambda size, spacing: spacing * size / 2, imdims, imspacing
    ))

    transform.SetFixedParameters(center)

    # Rotate
    image = resample_image(image, transform)

    if return_numpy:
        return convert_from_itk_image(image)
    else:
        return image


def resample_to_isotropic(itk_image):
    """
    This function can be used to rescale or upsample a confocal stack,
    which generally has a larger spacing in the z direction.

    :param itk_image:   an ITK:Image object
    :return:            returns a new ITK:Image object with rescaled
                        axial dimension
    """
    assert isinstance(itk_image, sitk.Image)

    method = sitk.ResampleImageFilter()
    transform = sitk.Transform()
    transform.SetIdentity()

    method.SetInterpolator(sitk.sitkBSpline)
    method.SetDefaultPixelValue(0)

    # Set output spacing
    spacing = itk_image.GetSpacing()

    if len(spacing) != 3:
        print("The function resample_to_isotropic(itk_image, image_type) is" \
              "intended for processing 3D images. The input image has %d " \
              "dimensions" % len(spacing))
        return

    scaling = spacing[2]/spacing[0]

    spacing[:] = spacing[0]
        
    method.SetOutputSpacing(spacing)
    method.SetOutputDirection(itk_image.GetDirection())
    method.SetOutputOrigin(itk_image.GetOrigin())

    # Set Output Image Size
    region = itk_image.GetLargestPossibleRegion()
    size = region.GetSize()
    size[2] = int(size[2]*scaling)
    method.SetSize(size)

    transform.SetIdentity()
    method.SetTransform(transform)

    return method.Execute(itk_image)


def rescale_intensity(image):
    """
    A filter to scale the intensities of the input image to the full range
    allowed by the pixel type

    Inputs:
        image       = an itk.Image() object
        input_type  = pixel type string of the input image. Must be an ITK
                      recognized pixel type
        output_type = same as above, for the output image
    """
    assert isinstance(image, sitk.Image)
    method = sitk.RescaleIntensityImageFilter()
    image_type = image.GetPixelIDTypeAsString()
    if image_type == '8-bit unsigned integer':
        method.SetOutputMinimum(0)
        method.SetOutputMaximum(255)
    else:    
        print("The rescale intensity filter has not been implemented for ", image_type)
        return image
    
    # TODO: Add pixel type check that is needed to check the bounds of re-scaling
    return method.Execute(image)


def gaussian_blurring_filter(image, variance):
    """
    Gaussian blur filter
    """

    filter = sitk.DiscreteGaussianImageFilter()
    filter.SetUseImageSpacing(False)
    filter.SetVariance(variance)

    return filter.Execute(image)


def grayscale_dilate_filter(image, kernel_radius):
    """
    Grayscale dilation filter
    """

    method = sitk.GrayscaleDilateImageFilter()
    kernel = method.GetKernel()
    kernel.SetKernelRadius(kernel_radius)
    kernel = kernel.Ball(kernel.GetRadius())
    method.SetKernel(kernel)

    return method.Execute(image)


def mean_filter(image, kernel_radius):
    """
    Uniform Mean filter for itk.Image objects
    """
    method = sitk.MeanImageFilter()
    method.SetRadius(kernel_radius)

    return method.Execute(image)


def median_filter(image, kernel_radius):
    """
    Median filter for itk.Image objects

    :param image:           an itk.Image object
    :param kernel_radius:   median kernel radius
    :return:                filtered image
    """
    method = sitk.MedianImageFilter()
    kernel = [kernel_radius,] * image.GetDimension()
    method.SetRadius(kernel)

    return method.Execute(image)


def normalize_image_filter(image):
    """
    Normalizes the pixel values in an image to Mean of zero and Variance
    of one. A floating point image_type is expected. For integer pixel
    type, casting to a float is recommended before using this.
    """

    method = sitk.NormalizeImageFilter()
    return method.Execute(image)


def threshold_image_filter(image, threshold, th_value=0,
                           th_method="below"):
    """
    Thresholds an image by setting pixel values above or below "threshold"
    to "th_value". The result is not a binary image, but a thresholded
    grayscale image.
    """

    method = sitk.ThresholdImageFilter()
    if th_method is "above":
        method.SetLower(threshold)
    elif th_method is "below":
        method.SetUpper(threshold)

    method.SetOutsideValue(th_value)

    return method.Execute(image)


def get_image_statistics(image):
    """
    A utility to calculate basic image statistics (Mean and Variance here)

    :param image:       an ITK:Image object
                        naming convention as in ITK
    :return:            returns the image mean and variance in a tuple
    """
    method = sitk.StatisticsImageFilter()
    method.Execute(image)
    mean = method.GetMean()
    variance = method.GetVariance()
    max = method.GetMaximum()
    min = method.GetMinimum()

    return mean, variance, min, max


def type_cast(image, output_type):
    """
    A utility for changing the image pixel container type

    :param image:       An ITK:Image
    :param output_type: output image type as ITK PixelID
    :return:            returns the image with new pixel type
    """
    assert isinstance(image, sitk.Image)

    method = sitk.CastImageFilter()
    method.SetOutputPixelType(output_type)

    return method.Execute(image)


def calculate_center_of_image(image, center_of_mass=False):
    """
    Center of an image can be defined either geometrically or statistically,
    as a Center-of-Gravity measure.

    This was originally Based on itk::ImageMomentsCalculator
    http://www.itk.org/Doxygen/html/classitk_1_1ImageMomentsCalculator.html

    However that filter is not currently implemented in SimpleITK and therefore
    a Numpy approach is used.
    """
    assert isinstance(image, sitk.Image)

    imdims = image.GetSize()
    imspacing = image.GetSpacing()

    if center_of_mass:
        np_image, spacing = convert_from_itk_image(image)
        center = scipy.ndimage.center_of_mass(np_image)
        center *= numpy.array(spacing)
    else:
        center = list(map(
            lambda size, spacing: spacing * size / 2,
            imdims, imspacing
        ))
    return center


def make_composite_rgb_image(red, green, blue=None, return_numpy=False):
    """
    A utitity to combine two or threegrayscale images into a single RGB image.
    If only two images are provided, an empty image is placed in the blue
    channel.

    :param red:  Red channel image. All the images should be sitk.Image
                 objects
    :param green Green channel image
    :param blue: Blue channel image.
    :return:     Returns a RGB composite image.
    """
    assert isinstance(red, sitk.Image) and isinstance(green, sitk.Image)
    red = sitk.Cast(red, sitk.sitkUInt8)
    green = sitk.Cast(green, sitk.sitkUInt8)
    if blue is not None:
        assert isinstance(blue, sitk.Image)
        return sitk.Compose(red, green, blue)
    else:
        blue = sitk.Image(red.GetSize(), sitk.sitkUInt8)
        blue.CopyInformation(red)
        if return_numpy:
            import numpy as np
            images = (convert_from_itk_image(red)[0],
                      convert_from_itk_image(green)[0],
                      convert_from_itk_image(blue)[0])

            spacing = convert_from_itk_image(red)[1]
            return np.concatenate(
                [aux[..., np.newaxis] for aux in images], axis=-1), spacing
        else:
            return sitk.Compose(red, green, blue)


def make_translation_transforms_from_xy(xs, ys):
    """
    Makes translation transforms from a set of (x,y) shift coordinate pairs
    :param xs: a list of x shifts
    :param ys: a list of y shifts
    :return: a list of sitk.TranslationTransforms
    """
    assert len(xs) == len(ys)
    transforms = []

    for x, y in zip(xs, ys):
        tfm = sitk.TranslationTransform(2)
        tfm.SetParameters((x, y))

        transforms.append(tfm)

    return transforms




















